==> bodo-2022.5.8-cp310-cp310-macosx_10_15_x86_64.whl: bodo/libs/tuple_arr_ext.py <==
"""Array of tuple values, implemented by reusing array of structs implementation.
"""
import operator
import numba
import numpy as np
from numba.core import types
from numba.extending import NativeValue, box, intrinsic, make_attribute_wrapper, models, overload, overload_attribute, overload_method, register_model, unbox
from numba.parfors.array_analysis import ArrayAnalysis
import bodo
from bodo.libs.struct_arr_ext import StructArrayType, box_struct_arr, unbox_struct_array


class TupleArrayType(types.ArrayCompatible):

    def __init__(self, data):
        self.data = data
        super(TupleArrayType, self).__init__(name='TupleArrayType({})'.
            format(data))

    @property
    def as_array(self):
        return types.Array(types.undefined, 1, 'C')

    @property
    def dtype(self):
        return types.BaseTuple.from_types(tuple(yxpr__kzghv.dtype for
            yxpr__kzghv in self.data))

    def copy(self):
        return TupleArrayType(self.data)

    @property
    def mangling_args(self):
        return self.__class__.__name__, (self._code,)


@register_model(TupleArrayType)
class TupleArrayModel(models.StructModel):

    def __init__(self, dmm, fe_type):
        aogkz__cvl = [('data', StructArrayType(fe_type.data))]
        models.StructModel.__init__(self, dmm, fe_type, aogkz__cvl)


make_attribute_wrapper(TupleArrayType, 'data', '_data')


@intrinsic
def init_tuple_arr(typingctx, data_typ=None):
    assert isinstance(data_typ, StructArrayType)
    fyh__pdso = TupleArrayType(data_typ.data)

    def codegen(context, builder, sig, args):
        mkm__muqa, = args
        xbrkd__wxj = context.make_helper(builder, fyh__pdso)
        xbrkd__wxj.data = mkm__muqa
        context.nrt.incref(builder, data_typ, mkm__muqa)
        return xbrkd__wxj._getvalue()
    return fyh__pdso(data_typ), codegen


@unbox(TupleArrayType)
def unbox_tuple_array(typ, val, c):
    data_typ = StructArrayType(typ.data)
    vrv__orijf = unbox_struct_array(data_typ, val, c, is_tuple_array=True)
    mkm__muqa = vrv__orijf.value
    xbrkd__wxj = c.context.make_helper(c.builder, typ)
    xbrkd__wxj.data = mkm__muqa
    sgkjf__kyphx = vrv__orijf.is_error
    return NativeValue(xbrkd__wxj._getvalue(), is_error=sgkjf__kyphx)


@box(TupleArrayType)
def box_tuple_arr(typ, val, c):
    data_typ = StructArrayType(typ.data)
    xbrkd__wxj = c.context.make_helper(c.builder, typ, val)
    arr = box_struct_arr(data_typ, xbrkd__wxj.data, c, is_tuple_array=True)
    return arr


@numba.njit
def pre_alloc_tuple_array(n, nested_counts, dtypes):
    return init_tuple_arr(bodo.libs.struct_arr_ext.pre_alloc_struct_array(n,
        nested_counts, dtypes, None))


def pre_alloc_tuple_array_equiv(self, scope, equiv_set, loc, args, kws):
    assert len(args) == 3 and not kws
    return ArrayAnalysis.AnalyzeResult(shape=args[0], pre=[])


(ArrayAnalysis._analyze_op_call_bodo_libs_tuple_arr_ext_pre_alloc_tuple_array
    ) = pre_alloc_tuple_array_equiv


@overload(operator.getitem, no_unliteral=True)
def tuple_arr_getitem(arr, ind):
    if not isinstance(arr, TupleArrayType):
        return
    if isinstance(ind, types.Integer):
        pgmf__tbyrz = 'def impl(arr, ind):\n'
        qkpy__zqcx = ','.join(f'get_data(arr._data)[{slihv__bzzsq}][ind]' for
            slihv__bzzsq in range(len(arr.data)))
        pgmf__tbyrz += f'  return ({qkpy__zqcx})\n'
        yjqt__njjsc = {}
        exec(pgmf__tbyrz, {'get_data': bodo.libs.struct_arr_ext.get_data},
            yjqt__njjsc)
        enl__mfmpx = yjqt__njjsc['impl']
        return enl__mfmpx

    def impl_arr(arr, ind):
        return init_tuple_arr(arr._data[ind])
    return impl_arr


@overload(operator.setitem, no_unliteral=True)
def tuple_arr_setitem(arr, ind, val):
    if not isinstance(arr, TupleArrayType):
        return
    if val == types.none or isinstance(val, types.optional):
        return
    if isinstance(ind, types.Integer):
        xcq__int = len(arr.data)
        pgmf__tbyrz = 'def impl(arr, ind, val):\n'
        pgmf__tbyrz += '  data = get_data(arr._data)\n'
        pgmf__tbyrz += '  null_bitmap = get_null_bitmap(arr._data)\n'
        pgmf__tbyrz += '  set_bit_to_arr(null_bitmap, ind, 1)\n'
        for slihv__bzzsq in range(xcq__int):
            pgmf__tbyrz += (
                f'  data[{slihv__bzzsq}][ind] = val[{slihv__bzzsq}]\n')
        yjqt__njjsc = {}
        exec(pgmf__tbyrz, {'get_data': bodo.libs.struct_arr_ext.get_data,
            'get_null_bitmap': bodo.libs.struct_arr_ext.get_null_bitmap,
            'set_bit_to_arr': bodo.libs.int_arr_ext.set_bit_to_arr},
            yjqt__njjsc)
        enl__mfmpx = yjqt__njjsc['impl']
        return enl__mfmpx

    def impl_arr(arr, ind, val):
        val = bodo.utils.conversion.coerce_to_array(val, use_nullable_array
            =True)
        arr._data[ind] = val._data
    return impl_arr


@overload(len, no_unliteral=True)
def overload_tuple_arr_len(A):
    if isinstance(A, TupleArrayType):
        return lambda A: len(A._data)


@overload_attribute(TupleArrayType, 'shape')
def overload_tuple_arr_shape(A):
    return lambda A: (len(A._data),)


@overload_attribute(TupleArrayType, 'dtype')
def overload_tuple_arr_dtype(A):
    return lambda A: np.object_


@overload_attribute(TupleArrayType, 'ndim')
def overload_tuple_arr_ndim(A):
    return lambda A: 1


@overload_attribute(TupleArrayType, 'nbytes')
def overload_tuple_arr_nbytes(A):
    return lambda A: A._data.nbytes


@overload_method(TupleArrayType, 'copy', no_unliteral=True)
def overload_tuple_arr_copy(A):

    def copy_impl(A):
        return init_tuple_arr(A._data.copy())
    return copy_impl

==> bodo-2022.7.1-cp310-cp310-macosx_10_15_x86_64.whl: bodo/libs/tuple_arr_ext.py <==
"""Array of tuple values, implemented by reusing array of structs implementation.
"""
import operator
import numba
import numpy as np
from numba.core import types
from numba.extending import NativeValue, box, intrinsic, make_attribute_wrapper, models, overload, overload_attribute, overload_method, register_model, unbox
from numba.parfors.array_analysis import ArrayAnalysis
import bodo
from bodo.libs.struct_arr_ext import StructArrayType, box_struct_arr, unbox_struct_array


class TupleArrayType(types.ArrayCompatible):

    def __init__(self, data):
        self.data = data
        super(TupleArrayType, self).__init__(name='TupleArrayType({})'.
            format(data))

    @property
    def as_array(self):
        return types.Array(types.undefined, 1, 'C')

    @property
    def dtype(self):
        return types.BaseTuple.from_types(tuple(paef__bnncp.dtype for
            paef__bnncp in self.data))

    def copy(self):
        return TupleArrayType(self.data)

    @property
    def mangling_args(self):
        return self.__class__.__name__, (self._code,)


@register_model(TupleArrayType)
class TupleArrayModel(models.StructModel):

    def __init__(self, dmm, fe_type):
        azjb__pwdvc = [('data', StructArrayType(fe_type.data))]
        models.StructModel.__init__(self, dmm, fe_type, azjb__pwdvc)


make_attribute_wrapper(TupleArrayType, 'data', '_data')


@intrinsic
def init_tuple_arr(typingctx, data_typ=None):
    assert isinstance(data_typ, StructArrayType)
    lpoj__eym = TupleArrayType(data_typ.data)

    def codegen(context, builder, sig, args):
        zpbq__lmmz, = args
        rvmka__ppr = context.make_helper(builder, lpoj__eym)
        rvmka__ppr.data = zpbq__lmmz
        context.nrt.incref(builder, data_typ, zpbq__lmmz)
        return rvmka__ppr._getvalue()
    return lpoj__eym(data_typ), codegen


@unbox(TupleArrayType)
def unbox_tuple_array(typ, val, c):
    data_typ = StructArrayType(typ.data)
    zarat__dezp = unbox_struct_array(data_typ, val, c, is_tuple_array=True)
    zpbq__lmmz = zarat__dezp.value
    rvmka__ppr = c.context.make_helper(c.builder, typ)
    rvmka__ppr.data = zpbq__lmmz
    csxo__jvfkj = zarat__dezp.is_error
    return NativeValue(rvmka__ppr._getvalue(), is_error=csxo__jvfkj)


@box(TupleArrayType)
def box_tuple_arr(typ, val, c):
    data_typ = StructArrayType(typ.data)
    rvmka__ppr = c.context.make_helper(c.builder, typ, val)
    arr = box_struct_arr(data_typ, rvmka__ppr.data, c, is_tuple_array=True)
    return arr


@numba.njit
def pre_alloc_tuple_array(n, nested_counts, dtypes):
    return init_tuple_arr(bodo.libs.struct_arr_ext.pre_alloc_struct_array(n,
        nested_counts, dtypes, None))


def pre_alloc_tuple_array_equiv(self, scope, equiv_set, loc, args, kws):
    assert len(args) == 3 and not kws
    return ArrayAnalysis.AnalyzeResult(shape=args[0], pre=[])


(ArrayAnalysis._analyze_op_call_bodo_libs_tuple_arr_ext_pre_alloc_tuple_array
    ) = pre_alloc_tuple_array_equiv


@overload(operator.getitem, no_unliteral=True)
def tuple_arr_getitem(arr, ind):
    if not isinstance(arr, TupleArrayType):
        return
    if isinstance(ind, types.Integer):
        tdln__oxbmp = 'def impl(arr, ind):\n'
        tfijq__azbt = ','.join(f'get_data(arr._data)[{mkh__fdsu}][ind]' for
            mkh__fdsu in range(len(arr.data)))
        tdln__oxbmp += f'  return ({tfijq__azbt})\n'
        dbvlg__jbsb = {}
        exec(tdln__oxbmp, {'get_data': bodo.libs.struct_arr_ext.get_data},
            dbvlg__jbsb)
        amwqx__uvtx = dbvlg__jbsb['impl']
        return amwqx__uvtx

    def impl_arr(arr, ind):
        return init_tuple_arr(arr._data[ind])
    return impl_arr


@overload(operator.setitem, no_unliteral=True)
def tuple_arr_setitem(arr, ind, val):
    if not isinstance(arr, TupleArrayType):
        return
    if val == types.none or isinstance(val, types.optional):
        return
    if isinstance(ind, types.Integer):
        iojzw__gakv = len(arr.data)
        tdln__oxbmp = 'def impl(arr, ind, val):\n'
        tdln__oxbmp += '  data = get_data(arr._data)\n'
        tdln__oxbmp += '  null_bitmap = get_null_bitmap(arr._data)\n'
        tdln__oxbmp += '  set_bit_to_arr(null_bitmap, ind, 1)\n'
        for mkh__fdsu in range(iojzw__gakv):
            tdln__oxbmp += f'  data[{mkh__fdsu}][ind] = val[{mkh__fdsu}]\n'
        dbvlg__jbsb = {}
        exec(tdln__oxbmp, {'get_data': bodo.libs.struct_arr_ext.get_data,
            'get_null_bitmap': bodo.libs.struct_arr_ext.get_null_bitmap,
            'set_bit_to_arr': bodo.libs.int_arr_ext.set_bit_to_arr},
            dbvlg__jbsb)
        amwqx__uvtx = dbvlg__jbsb['impl']
        return amwqx__uvtx

    def impl_arr(arr, ind, val):
        val = bodo.utils.conversion.coerce_to_array(val, use_nullable_array
            =True)
        arr._data[ind] = val._data
    return impl_arr


@overload(len, no_unliteral=True)
def overload_tuple_arr_len(A):
    if isinstance(A, TupleArrayType):
        return lambda A: len(A._data)


@overload_attribute(TupleArrayType, 'shape')
def overload_tuple_arr_shape(A):
    return lambda A: (len(A._data),)


@overload_attribute(TupleArrayType, 'dtype')
def overload_tuple_arr_dtype(A):
    return lambda A: np.object_


@overload_attribute(TupleArrayType, 'ndim')
def overload_tuple_arr_ndim(A):
    return lambda A: 1


@overload_attribute(TupleArrayType, 'nbytes')
def overload_tuple_arr_nbytes(A):
    return lambda A: A._data.nbytes


@overload_method(TupleArrayType, 'copy', no_unliteral=True)
def overload_tuple_arr_copy(A):

    def copy_impl(A):
        return init_tuple_arr(A._data.copy())
    return copy_impl
